from flask import abort, request, jsonify
import datetime, os, time
from app.video_cut.autocut import wmdigit_cut, wmdigit_transcribe, utils


def error(msg):
    abort(jsonify({"error": msg}))


# 校验请求参数
def validate_request():
    if not request.json or 'instances' not in request.json:
        error("参数错误: 缺少instances参数")

    instance = request.json['instances']
    if len(instance) <= 0 or 'video' not in instance[0] or 'steps' not in instance[0] or 'lang' not in instance[0]:
        error('参数错误: instances需要: video, steps, lang')
    video = instance[0]['video']
    steps = instance[0]['steps']
    lang = instance[0]['lang']
    if len(video) <= 0:
        error('参数错误: video 参数不可为空')
    if len(steps) <= 0 or ('step1' not in steps and 'step2' not in steps):
        error('参数错误: steps 参数不可为空，取值为：step1、step2')
    if len(lang) <= 0:
            error('参数错误: lang 参数不可为空')

    with_sub = instance[0]['with_sub'] if 'with_sub' in instance[0] and instance[0]['with_sub'] else False

    srt_json_in = ""
    if len(steps) == 1 and 'step2' in steps:
        if 'srt_json_in' not in instance[0]:
            error('参数错误: instances需要: srt_json_in')
        srt_json_in = instance[0]['srt_json_in']
        if len(srt_json_in) <= 0:
            error('参数错误: srt_json_in 参数不可为空')

    return video, steps, lang, with_sub, srt_json_in


# 主线
def video_cut_pipeline(logger, args, whispermodel):
    # print(args)
    time_record = []
    media_file, steps, lang, with_sub, srt_json_in = validate_request()
    all_start_time = time.time()
    args.lang = lang
    srt_json_out = ""
    final_video_fn = ""
    if 'step1' in steps:
        # 1、从视频生成字幕
        time_record, srt_json_out = step1(time_record, logger, args, whispermodel, media_file, lang)
    if 'step2' in steps:
        srt_json = srt_json_out if 'step1' in steps else srt_json_in
        # 2、从字幕生成cut视频
        time_record, final_video_fn = step2(time_record, logger, args, media_file, with_sub, srt_json)
    time_record.append(f"所有步骤处理完毕。耗时: {time.time() - all_start_time:.4f} 秒")
    for i in time_record:
        print(i)
    # 如果只有step1则返回字幕，如果只有step2或者1和2都有，则返回最终结果
    if len(steps) == 1 and steps[0] == 'step1':
        return srt_json_out
    else:
        return final_video_fn


def step1(time_record, logger, args, whispermodel, media_file, lang):
    start_time = time.time()
    if lang != "zh":
        prompt = f"Subtitles must be fully translated into {lang}"
    else:
        prompt = ""
    logger.info(f"Transcribe {media_file} lang={lang} promt={prompt}")
    args.inputs = [media_file]
    md_fn, srt_fn, srt_json_out = wmdigit_transcribe.Transcribe(args, whispermodel).run()
    time_record.append(f"视频生成srt和md。耗时: {time.time() - start_time:.4f} 秒")
    return time_record, srt_json_out


def step2(time_record, logger, args, media_file, with_sub, srt_json_in):
    start_time = time.time()
    srt_fn = utils.change_ext(media_file, "srt")
    md_fn = utils.change_ext(media_file, "md")
    # 根据 srt_json_in 重新生成 srt 文件
    utils.gen_srt_from_subjson(srt_json_in, srt_fn)
    logger.info(f"Cut {media_file} srt={srt_fn} sub={with_sub}")
    args.inputs = [media_file, md_fn, srt_fn]
    final_video_fn, new_srt_fn = wmdigit_cut.Cutter(args).run()
    time_record.append(f"从字幕生成cut视频。耗时: {time.time() - start_time:.4f} 秒")
    # 加字幕
    if with_sub:
        start_time = time.time()
        final_video_fn = utils.combine_video_with_subtitle(final_video_fn, new_srt_fn)
        time_record.append(f"加字幕耗时: {time.time() - start_time:.4f} 秒")
    return time_record, final_video_fn